@@ -1,11 +1,11 @@
-/* $Id: display.c,v 1.3 2001/11/15 00:32:13 a-ito Exp $ */
+/* $Id: display.c,v 1.12 2001/12/04 16:33:08 ukai Exp $ */
 #include <signal.h>
 #include "fm.h"
 
 #define MAX(a, b)  ((a) > (b) ? (a) : (b))
 #define MIN(a, b)  ((a) < (b) ? (a) : (b))
 
-#ifdef COLOR
+#ifdef USE_COLOR
 
 #define EFFECT_ANCHOR_START       effect_anchor_start()
 #define EFFECT_ANCHOR_END         effect_anchor_end()
@@ -43,17 +43,22 @@
 #define EFFECT_ACTIVE_END_NC        underlineend()
 #define EFFECT_ACTIVE_START_M       bold()
 #define EFFECT_ACTIVE_END_M         boldend()
-#define EFFECT_VISITED_START_M      /**/
-#define EFFECT_VISITED_END_M        /**/
-
+#define EFFECT_VISITED_START_M /**/
+#define EFFECT_VISITED_END_M /**/
 #define define_effect(name_start,name_end,color_start,color_end,mono_start,mono_end) \
 static void name_start { if (useColor) { color_start; } else { mono_start; }}\
 static void name_end { if (useColor) { color_end; } else { mono_end; }}
+define_effect(EFFECT_ANCHOR_START, EFFECT_ANCHOR_END, EFFECT_ANCHOR_START_C,
+	      EFFECT_ANCHOR_END_C, EFFECT_ANCHOR_START_M, EFFECT_ANCHOR_END_M)
+define_effect(EFFECT_IMAGE_START, EFFECT_IMAGE_END, EFFECT_IMAGE_START_C,
+	      EFFECT_IMAGE_END_C, EFFECT_IMAGE_START_M, EFFECT_IMAGE_END_M)
+define_effect(EFFECT_FORM_START, EFFECT_FORM_END, EFFECT_FORM_START_C,
+	      EFFECT_FORM_END_C, EFFECT_FORM_START_M, EFFECT_FORM_END_M)
 
-define_effect(EFFECT_ANCHOR_START, EFFECT_ANCHOR_END, EFFECT_ANCHOR_START_C, EFFECT_ANCHOR_END_C, EFFECT_ANCHOR_START_M, EFFECT_ANCHOR_END_M)
-define_effect(EFFECT_IMAGE_START, EFFECT_IMAGE_END, EFFECT_IMAGE_START_C, EFFECT_IMAGE_END_C, EFFECT_IMAGE_START_M, EFFECT_IMAGE_END_M)
-define_effect(EFFECT_FORM_START, EFFECT_FORM_END, EFFECT_FORM_START_C, EFFECT_FORM_END_C, EFFECT_FORM_START_M, EFFECT_FORM_END_M)
-static void EFFECT_ACTIVE_START
+/*****************/
+/* *INDENT-OFF* */
+static void
+EFFECT_ACTIVE_START
 {
     if (useColor) {
 	if (useActiveColor) {
@@ -73,7 +78,8 @@ static void EFFECT_ACTIVE_START
     }
 }
 
-static void EFFECT_ACTIVE_END
+static void
+EFFECT_ACTIVE_END
 {
     if (useColor) {
 	if (useActiveColor) {
@@ -82,11 +88,12 @@ static void EFFECT_ACTIVE_END
 	    EFFECT_ACTIVE_END_NC;
 	}
     } else {
-	 EFFECT_ACTIVE_END_M;
+	EFFECT_ACTIVE_END_M;
     }
 }
 
-static void EFFECT_VISITED_START
+static void
+EFFECT_VISITED_START
 {
     if (useVisitedColor) {
 	if (useColor) {
@@ -97,7 +104,8 @@ static void EFFECT_VISITED_START
     }
 }
 
-static void EFFECT_VISITED_END
+static void
+EFFECT_VISITED_END
 {
     if (useVisitedColor) {
 	if (useColor) {
@@ -107,8 +115,8 @@ static void EFFECT_VISITED_END
 	}
     }
 }
-
-#else				/* not COLOR */
+/* *INDENT-ON* */
+#else				/* not USE_COLOR */
 
 #define EFFECT_ANCHOR_START       underline()
 #define EFFECT_ANCHOR_END         underlineend()
@@ -118,12 +126,9 @@ static void EFFECT_VISITED_END
 #define EFFECT_FORM_END           standend()
 #define EFFECT_ACTIVE_START       bold()
 #define EFFECT_ACTIVE_END         boldend()
-#define EFFECT_VISITED_START      /**/
-#define EFFECT_VISITED_END        /**/
-
-#endif				/* not COLOR */
-
-
+#define EFFECT_VISITED_START /**/
+#define EFFECT_VISITED_END /**/
+#endif				/* not USE_COLOR */
 #ifndef KANJI_SYMBOLS
 static char g_rule[] = "ntwluxkavmqajaaa";
 #endif				/* not KANJI_SYMBOLS */
@@ -139,10 +144,10 @@ fmTerm(void)
 	move(LASTLINE, 0);
 	clrtoeolx();
 	refresh();
-#ifdef MOUSE
+#ifdef USE_MOUSE
 	if (use_mouse)
 	    mouse_end();
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 	reset_tty();
 	fmInitialized = FALSE;
     }
@@ -157,9 +162,6 @@ fmInit(void)
 {
     if (!fmInitialized) {
 	initscr();
-#if defined( __CYGWIN__ ) && defined( JP_CHARSET )
-	init_win32_console_handle();
-#endif
 	term_raw();
 	term_noecho();
     }
@@ -174,22 +176,24 @@ static int ccolumn = -1;
 
 static int ulmode = 0, somode = 0, bomode = 0;
 static int anch_mode = 0, emph_mode = 0, imag_mode = 0, form_mode = 0,
- active_mode = 0, visited_mode = 0;
+    active_mode = 0, visited_mode = 0;
 #ifndef KANJI_SYMBOLS
 static int graph_mode = 0;
 #endif				/* not KANJI_SYMBOLS */
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 static Linecolor color_mode = 0;
 #endif
 
-#ifdef BUFINFO
+#ifdef USE_BUFINFO
 static Buffer *save_current_buf = NULL;
 #endif
 
 int in_check_url = FALSE;
 
+char *delayed_msg = NULL;
+
 void
-displayBuffer(Buffer * buf, int mode)
+displayBuffer(Buffer *buf, int mode)
 {
     Str msg;
     Anchor *aa = NULL;
@@ -204,15 +208,27 @@ displayBuffer(Buffer * buf, int mode)
 	buf->width = COLS;
     if (buf->height == 0)
 	buf->height = LASTLINE + 1;
-    if (buf->width != INIT_BUFFER_WIDTH && buf->type && !strcmp(buf->type, "text/html")) {
+    if (buf->width != INIT_BUFFER_WIDTH && buf->type
+	&& !strcmp(buf->type, "text/html")) {
 	in_check_url = TRUE;
 	reshapeBuffer(buf);
 	in_check_url = FALSE;
     }
+    if (showLineNum) {
+	if (buf->lastLine && buf->lastLine->real_linenumber > 0)
+	    buf->rootX = (int)(log(buf->lastLine->real_linenumber + 0.1)
+			       / log(10)) + 2;
+	if (buf->rootX < 5)
+	    buf->rootX = 5;
+	if (buf->rootX > COLS)
+	    buf->rootX = COLS;
+    }
+    else
+	buf->rootX = 0;
+    buf->COLS = COLS - buf->rootX;
     if (mode == B_FORCE_REDRAW ||
 	mode == B_SCROLL ||
-	cline != buf->topLine ||
-	ccolumn != buf->currentColumn) {
+	cline != buf->topLine || ccolumn != buf->currentColumn) {
 	if (mode == B_SCROLL && cline && buf->currentColumn == ccolumn) {
 	    int n = buf->topLine->linenumber - cline->linenumber;
 	    if (n > 0 && n < LASTLINE) {
@@ -222,11 +238,11 @@ displayBuffer(Buffer * buf, int mode)
 		scroll(n);
 	    }
 	    else if (n < 0 && n > -LASTLINE) {
-#if defined(CYGWIN) && LANG == JA
+#if defined(__CYGWIN__) && LANG == JA
 		move(LASTLINE + n + 1, 0);
 		clrtoeolx();
 		refresh();
-#endif				/* defined(CYGWIN) && LANG == JA */
+#endif				/* defined(__CYGWIN__) && LANG == JA */
 		rscroll(-n);
 	    }
 	    redrawNLine(buf, n);
@@ -239,7 +255,7 @@ displayBuffer(Buffer * buf, int mode)
     if (buf->topLine == NULL)
 	buf->topLine = buf->firstLine;
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
     if (use_mouse)
 #if LANG == JA
 	msg = Strnew_charp("㢬");
@@ -247,7 +263,7 @@ displayBuffer(Buffer * buf, int mode)
 	msg = Strnew_charp("<=UpDn ");
 #endif				/* LANG != JA */
     else
-#endif				/* not MOUSE */
+#endif				/* not USE_MOUSE */
 	msg = Strnew();
     Strcat_charp(msg, "Viewing <");
     Strcat_charp(msg, buf->buffername);
@@ -270,10 +286,12 @@ displayBuffer(Buffer * buf, int mode)
 #endif				/* LANG != JA */
 		l = buf->width - msg->length;
 		Strcat(msg, Strsubstr(s, s->length - l, l));
-	    } else {
+	    }
+	    else {
 		msg = s;
 	    }
-	} else {
+	}
+	else {
 	    l -= s->length;
 	    if (msg->length > l) {
 #ifdef JP_CHARSET
@@ -291,18 +309,24 @@ displayBuffer(Buffer * buf, int mode)
 	    Strcat_charp(msg, "> ");
 	    Strcat(msg, s);
 	}
-    } else {
+    }
+    else {
 	Strcat_charp(msg, ">");
     }
     if (buf->firstLine == NULL) {
 	Strcat_charp(msg, "\tNo Line");
 	clear();
     }
+    if (delayed_msg != NULL) {
+	disp_message(delayed_msg, FALSE);
+	delayed_msg = NULL;
+	refresh();
+    }
     standout();
-    message(msg->ptr, buf->cursorX, buf->cursorY);
+    message(msg->ptr, buf->cursorX + buf->rootX, buf->cursorY);
     standend();
     refresh();
-#ifdef BUFINFO
+#ifdef USE_BUFINFO
     if (Currentbuf != save_current_buf) {
 	saveBufferInfo();
 	save_current_buf = Currentbuf;
@@ -311,25 +335,25 @@ displayBuffer(Buffer * buf, int mode)
 }
 
 void
-redrawBuffer(Buffer * buf)
+redrawBuffer(Buffer *buf)
 {
     redrawNLine(buf, LASTLINE);
 }
 
 void
-redrawNLine(Buffer * buf, int n)
+redrawNLine(Buffer *buf, int n)
 {
     Line *l, *l0;
     int i;
 
-#ifdef COLOR
+#ifdef USE_COLOR
     if (useColor) {
 	EFFECT_ANCHOR_END_C;
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
 	setbcolor(bg_color);
-#endif				/* BG_COLOR */
+#endif				/* USE_BG_COLOR */
     }
-#endif				/* COLOR */
+#endif				/* USE_COLOR */
     for (i = 0, l = buf->topLine; i < LASTLINE; i++) {
 	if (i >= LASTLINE - n || i < -n)
 	    l0 = redrawLine(buf, l, i);
@@ -347,16 +371,16 @@ redrawNLine(Buffer * buf, int n)
 #define addKanji(pc,pr) (addChar((pc)[0],(pr)[0]),addChar((pc)[1],(pr)[1]))
 
 Line *
-redrawLine(Buffer * buf, Line * l, int i)
+redrawLine(Buffer *buf, Line *l, int i)
 {
     int j, pos, rcol, ncol, delta;
     int column = buf->currentColumn;
     char *p;
     Lineprop *pr;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     Linecolor *pc;
 #endif
-#ifdef COLOR
+#ifdef USE_COLOR
     Anchor *a;
     ParsedURL url;
     int k, vpos = -1;
@@ -372,6 +396,25 @@ redrawLine(Buffer * buf, Line * l, int i)
 	    return NULL;
     }
     move(i, 0);
+    if (showLineNum) {
+	char tmp[16];
+	if (!buf->rootX) {
+	    if (buf->lastLine->real_linenumber > 0)
+		buf->rootX = (int)(log(buf->lastLine->real_linenumber + 0.1)
+				   / log(10)) + 2;
+	    if (buf->rootX < 5)
+		buf->rootX = 5;
+	    if (buf->rootX > COLS)
+		buf->rootX = COLS;
+	    buf->COLS = COLS - buf->rootX;
+	}
+	if (l->real_linenumber)
+	    sprintf(tmp, "%*ld:", buf->rootX - 1, l->real_linenumber);
+	else
+	    sprintf(tmp, "%*s ", buf->rootX - 1, "");
+	addstr(tmp);
+    }
+    move(i, buf->rootX);
     if (l->width < 0)
 	l->width = COLPOS(l, l->len);
     if (l->len == 0 || l->width - 1 < column) {
@@ -382,7 +425,7 @@ redrawLine(Buffer * buf, Line * l, int i)
     pos = columnPos(l, column);
     p = &(l->lineBuf[pos]);
     pr = &(l->propBuf[pos]);
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     if (useColor && l->colorBuf)
 	pc = &(l->colorBuf[pos]);
     else
@@ -393,8 +436,8 @@ redrawLine(Buffer * buf, Line * l, int i)
 #ifndef JP_CHARSET
     delta = 1;
 #endif
-    for (j = 0; rcol - column < COLS && pos + j < l->len; j += delta) {
-#ifdef COLOR
+    for (j = 0; rcol - column < buf->COLS && pos + j < l->len; j += delta) {
+#ifdef USE_COLOR
 	if (useVisitedColor && vpos <= pos + j && !(pr[j] & PE_VISITED)) {
 	    a = retrieveAnchor(buf->href, l->linenumber, pos + j);
 	    if (a) {
@@ -414,9 +457,9 @@ redrawLine(Buffer * buf, Line * l, int i)
 	    delta = 1;
 #endif
 	ncol = COLPOS(l, pos + j + delta);
-	if (ncol - column > COLS)
+	if (ncol - column > buf->COLS)
 	    break;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	if (pc)
 	    do_color(pc[j]);
 #endif
@@ -482,27 +525,27 @@ redrawLine(Buffer * buf, Line * l, int i)
 	graphend();
     }
 #endif				/* not KANJI_SYMBOLS */
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     if (color_mode)
 	do_color(0);
 #endif
-    if (rcol - column < COLS)
+    if (rcol - column < buf->COLS)
 	clrtoeolx();
     return l->next;
 }
 
 int
-redrawLineRegion(Buffer * buf, Line * l, int i, int bpos, int epos)
+redrawLineRegion(Buffer *buf, Line *l, int i, int bpos, int epos)
 {
     int j, pos, rcol, ncol, delta;
     int column = buf->currentColumn;
     char *p;
     Lineprop *pr;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     Linecolor *pc;
 #endif
     int bcol, ecol;
-#ifdef COLOR
+#ifdef USE_COLOR
     Anchor *a;
     ParsedURL url;
     int k, vpos = -1;
@@ -513,7 +556,7 @@ redrawLineRegion(Buffer * buf, Line * l, int i, int bpos, int epos)
     pos = columnPos(l, column);
     p = &(l->lineBuf[pos]);
     pr = &(l->propBuf[pos]);
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     if (useColor && l->colorBuf)
 	pc = &(l->colorBuf[pos]);
     else
@@ -526,8 +569,8 @@ redrawLineRegion(Buffer * buf, Line * l, int i, int bpos, int epos)
 #ifndef JP_CHARSET
     delta = 1;
 #endif
-    for (j = 0; rcol - column < COLS && pos + j < l->len; j += delta) {
-#ifdef COLOR
+    for (j = 0; rcol - column < buf->COLS && pos + j < l->len; j += delta) {
+#ifdef USE_COLOR
 	if (useVisitedColor && vpos <= pos + j && !(pr[j] & PE_VISITED)) {
 	    a = retrieveAnchor(buf->href, l->linenumber, pos + j);
 	    if (a) {
@@ -547,20 +590,20 @@ redrawLineRegion(Buffer * buf, Line * l, int i, int bpos, int epos)
 	    delta = 1;
 #endif
 	ncol = COLPOS(l, pos + j + delta);
-	if (ncol - column > COLS)
+	if (ncol - column > buf->COLS)
 	    break;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	if (pc)
 	    do_color(pc[j]);
 #endif
 	if (j >= bcol && j < ecol) {
 	    if (rcol < column) {
-		move(i, 0);
+		move(i, buf->rootX);
 		for (rcol = column; rcol < ncol; rcol++)
 		    addChar(' ', 0);
 		continue;
 	    }
-	    move(i, rcol - column);
+	    move(i, rcol - column + buf->rootX);
 	    if (p[j] == '\t') {
 		for (; rcol < ncol; rcol++)
 		    addChar(' ', 0);
@@ -619,7 +662,7 @@ redrawLineRegion(Buffer * buf, Line * l, int i, int bpos, int epos)
 	graphend();
     }
 #endif				/* not KANJI_SYMBOLS */
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     if (color_mode)
 	do_color(0);
 #endif
@@ -651,7 +694,8 @@ do_effects(Lineprop m)
     do_effect2(PE_ANCHOR, anch_mode, EFFECT_ANCHOR_START, EFFECT_ANCHOR_END);
     do_effect2(PE_IMAGE, imag_mode, EFFECT_IMAGE_START, EFFECT_IMAGE_END);
     do_effect2(PE_FORM, form_mode, EFFECT_FORM_START, EFFECT_FORM_END);
-    do_effect2(PE_VISITED, visited_mode, EFFECT_VISITED_START, EFFECT_VISITED_END);
+    do_effect2(PE_VISITED, visited_mode, EFFECT_VISITED_START,
+	       EFFECT_VISITED_END);
     do_effect2(PE_ACTIVE, active_mode, EFFECT_ACTIVE_START, EFFECT_ACTIVE_END);
 #ifndef KANJI_SYMBOLS
     if (graph_mode) {
@@ -668,7 +712,8 @@ do_effects(Lineprop m)
     do_effect1(PE_ANCHOR, anch_mode, EFFECT_ANCHOR_START, EFFECT_ANCHOR_END);
     do_effect1(PE_IMAGE, imag_mode, EFFECT_IMAGE_START, EFFECT_IMAGE_END);
     do_effect1(PE_FORM, form_mode, EFFECT_FORM_START, EFFECT_FORM_END);
-    do_effect1(PE_VISITED, visited_mode, EFFECT_VISITED_START, EFFECT_VISITED_END);
+    do_effect1(PE_VISITED, visited_mode, EFFECT_VISITED_START,
+	       EFFECT_VISITED_END);
     do_effect1(PE_ACTIVE, active_mode, EFFECT_ACTIVE_START, EFFECT_ACTIVE_END);
 #ifndef KANJI_SYMBOLS
     if (m & PC_RULE) {
@@ -680,7 +725,7 @@ do_effects(Lineprop m)
 #endif				/* not KANJI_SYMBOLS */
 }
 
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 void
 do_color(Linecolor c)
 {
@@ -688,7 +733,7 @@ do_color(Linecolor c)
 	setfcolor(c & 0x7);
     else if (color_mode & 0x8)
 	setfcolor(basic_color);
-#ifdef BG_COLOR
+#ifdef USE_BG_COLOR
     if (c & 0x80)
 	setbcolor((c >> 4) & 0x7);
     else if (color_mode & 0x80)
@@ -706,14 +751,15 @@ addChar(char c, Lineprop mode)
 #ifdef JP_CHARSET
     if (CharType(mode) != PC_KANJI2)
 #endif				/* JP_CHARSET */
-    do_effects(m);
+	do_effects(m);
 #ifndef KANJI_SYMBOLS
     if (m & PC_RULE) {
 	if (graph_mode)
 	    addch(g_rule[c & 0xF]);
 	else
 	    addch(alt_rule[c & 0xF]);
-    } else
+    }
+    else
 #endif				/* not KANJI_SYMBOLS */
     if (IS_UNPRINTABLE_ASCII(c, mode)) {
 	addstr(Sprintf("\\%3o", (unsigned char)c)->ptr);
@@ -729,7 +775,7 @@ addChar(char c, Lineprop mode)
     }
     else if (c != '\n')
 	addch(c);
-    else	/* \n */
+    else			/* \n */
 	addch(' ');
 }
 
@@ -743,7 +789,7 @@ record_err_message(char *s)
 	    message_list = newGeneralList();
 	if (message_list->nitem >= LINES)
 	    popValue(message_list);
-	pushValue(message_list, allocStr(s, 0));
+	pushValue(message_list, allocStr(s, -1));
     }
 }
 
@@ -760,10 +806,11 @@ message_list_panel(void)
 		 "<html><head><title>List of error messages</title></head><body>"
 		 "<h1>List of error messages</h1><table cellpadding=0>\n");
     if (message_list)
-      for (p = message_list->last ; p ; p = p->prev)
-	Strcat_m_charp(tmp, "<tr><td><pre>", html_quote(p->ptr), "</pre></td></tr>\n", NULL);
+	for (p = message_list->last; p; p = p->prev)
+	    Strcat_m_charp(tmp, "<tr><td><pre>", html_quote(p->ptr),
+			   "</pre></td></tr>\n", NULL);
     else
-      Strcat_charp(tmp, "<tr><td>(no message recorded)</td></tr>\n");
+	Strcat_charp(tmp, "<tr><td>(no message recorded)</td></tr>\n");
     Strcat_charp(tmp, "</table></body></html>");
     return loadHTMLString(tmp);
 }
@@ -787,16 +834,17 @@ disp_message_nsec(char *s, int redraw_current, int sec, int purge, int mouse)
 	return;
     }
     if (Currentbuf != NULL)
-	message(s, Currentbuf->cursorX, Currentbuf->cursorY);
+	message(s, Currentbuf->cursorX + Currentbuf->rootX,
+		Currentbuf->cursorY);
     else
 	message(s, LASTLINE, 0);
     refresh();
-#ifdef MOUSE
+#ifdef USE_MOUSE
     if (mouse && use_mouse)
 	mouse_active();
 #endif
     sleep_till_anykey(sec, purge);
-#ifdef MOUSE
+#ifdef USE_MOUSE
     if (mouse && use_mouse)
 	mouse_inactive();
 #endif
@@ -809,7 +857,7 @@ disp_message(char *s, int redraw_current)
 {
     disp_message_nsec(s, redraw_current, 10, FALSE, TRUE);
 }
-#ifdef MOUSE
+#ifdef USE_MOUSE
 void
 disp_message_nomouse(char *s, int redraw_current)
 {
@@ -818,14 +866,20 @@ disp_message_nomouse(char *s, int redraw_current)
 #endif
 
 void
-cursorUp(Buffer * buf, int n)
+set_delayed_message(char *s)
+{
+    delayed_msg = allocStr(s, -1);
+}
+
+void
+cursorUp(Buffer *buf, int n)
 {
     if (buf->firstLine == NULL)
 	return;
     if (buf->cursorY > 0)
 	cursorUpDown(buf, -1);
     else {
-       buf->topLine = lineSkip(buf, buf->topLine, - n, FALSE);
+	buf->topLine = lineSkip(buf, buf->topLine, -n, FALSE);
 	if (buf->currentLine->prev != NULL)
 	    buf->currentLine = buf->currentLine->prev;
 	arrangeLine(buf);
@@ -833,14 +887,14 @@ cursorUp(Buffer * buf, int n)
 }
 
 void
-cursorDown(Buffer * buf, int n)
+cursorDown(Buffer *buf, int n)
 {
     if (buf->firstLine == NULL)
 	return;
     if (buf->cursorY < LASTLINE - 1)
 	cursorUpDown(buf, 1);
     else {
-       buf->topLine = lineSkip(buf, buf->topLine, n, FALSE);
+	buf->topLine = lineSkip(buf, buf->topLine, n, FALSE);
 	if (buf->currentLine->next != NULL)
 	    buf->currentLine = buf->currentLine->next;
 	arrangeLine(buf);
@@ -848,7 +902,7 @@ cursorDown(Buffer * buf, int n)
 }
 
 void
-cursorUpDown(Buffer * buf, int n)
+cursorUpDown(Buffer *buf, int n)
 {
     Line *cl = buf->currentLine;
 
@@ -860,7 +914,7 @@ cursorUpDown(Buffer * buf, int n)
 }
 
 void
-cursorRight(Buffer * buf, int n)
+cursorRight(Buffer *buf, int n)
 {
     int i, delta = 1, cpos, vpos2;
     Line *l = buf->currentLine;
@@ -880,32 +934,32 @@ cursorRight(Buffer * buf, int n)
 	buf->pos = i + delta;
     }
     else if (l->len == 0) {
-        buf->pos = 0;
+	buf->pos = 0;
     }
     else {
-        buf->pos = l->len -1;
+	buf->pos = l->len - 1;
 #ifdef JP_CHARSET
-        if (CharType(p[buf->pos]) == PC_KANJI2)
-            buf->pos--;
-#endif                          /* JP_CHARSET */
+	if (CharType(p[buf->pos]) == PC_KANJI2)
+	    buf->pos--;
+#endif				/* JP_CHARSET */
     }
     cpos = COLPOS(l, buf->pos);
     buf->visualpos = cpos - buf->currentColumn;
     delta = 1;
 #ifdef JP_CHARSET
     if (CharType(p[buf->pos]) == PC_KANJI1)
-        delta = 2;
-#endif                          /* JP_CHARSET */
+	delta = 2;
+#endif				/* JP_CHARSET */
     vpos2 = COLPOS(l, buf->pos + delta) - buf->currentColumn - 1;
-    if (vpos2 >= COLS && n) {
-       columnSkip(buf, n + (vpos2 - COLS) - (vpos2 - COLS) % n);
-       buf->visualpos = cpos - buf->currentColumn;
+    if (vpos2 >= buf->COLS && n) {
+	columnSkip(buf, n + (vpos2 - buf->COLS) - (vpos2 - buf->COLS) % n);
+	buf->visualpos = cpos - buf->currentColumn;
     }
     buf->cursorX = buf->visualpos;
 }
 
 void
-cursorLeft(Buffer * buf, int n)
+cursorLeft(Buffer *buf, int n)
 {
     int i, delta = 1, cpos;
     Line *l = buf->currentLine;
@@ -926,14 +980,14 @@ cursorLeft(Buffer * buf, int n)
     cpos = COLPOS(l, buf->pos);
     buf->visualpos = cpos - buf->currentColumn;
     if (buf->visualpos < 0 && n) {
-       columnSkip(buf, - n + buf->visualpos - buf->visualpos % n);
+	columnSkip(buf, -n + buf->visualpos - buf->visualpos % n);
 	buf->visualpos = cpos - buf->currentColumn;
     }
     buf->cursorX = buf->visualpos;
 }
 
 void
-cursorHome(Buffer * buf)
+cursorHome(Buffer *buf)
 {
     buf->visualpos = 0;
     buf->cursorX = buf->cursorY = 0;
@@ -945,9 +999,9 @@ cursorHome(Buffer * buf)
  * current position.
  */
 void
-arrangeCursor(Buffer * buf)
+arrangeCursor(Buffer *buf)
 {
-    int col,col2;
+    int col, col2;
     int delta = 1;
     if (buf == NULL || buf->currentLine == NULL)
 	return;
@@ -968,26 +1022,28 @@ arrangeCursor(Buffer * buf)
     col = COLPOS(buf->currentLine, buf->pos);
 #ifdef JP_CHARSET
     if (CharType(buf->currentLine->propBuf[buf->pos]) == PC_KANJI1)
-        delta = 2;
-#endif                         /* JP_CHARSET */
+	delta = 2;
+#endif				/* JP_CHARSET */
     col2 = COLPOS(buf->currentLine, buf->pos + delta);
-    if (col < buf->currentColumn || col2 > COLS + buf->currentColumn) {
+    if (col < buf->currentColumn || col2 > buf->COLS + buf->currentColumn) {
 	buf->currentColumn = 0;
-        if (col2 > COLS)
+	if (col2 > buf->COLS)
 	    columnSkip(buf, col);
     }
     /* Arrange cursor */
     buf->cursorY = buf->currentLine->linenumber - buf->topLine->linenumber;
-    buf->visualpos = buf->cursorX = COLPOS(buf->currentLine, buf->pos) - buf->currentColumn;
+    buf->visualpos = buf->cursorX =
+	COLPOS(buf->currentLine, buf->pos) - buf->currentColumn;
 #ifdef DISPLAY_DEBUG
-    fprintf(stderr, "arrangeCursor: column=%d, cursorX=%d, visualpos=%d, pos=%d, len=%d\n",
-	    buf->currentColumn, buf->cursorX, buf->visualpos,
-	    buf->pos, buf->currentLine->len);
+    fprintf(stderr,
+	    "arrangeCursor: column=%d, cursorX=%d, visualpos=%d, pos=%d, len=%d\n",
+	    buf->currentColumn, buf->cursorX, buf->visualpos, buf->pos,
+	    buf->currentLine->len);
 #endif
 }
 
 void
-arrangeLine(Buffer * buf)
+arrangeLine(Buffer *buf)
 {
     int i, cpos;
 
@@ -1017,14 +1073,15 @@ arrangeLine(Buffer * buf)
 	buf->pos = 0;
     }
 #ifdef DISPLAY_DEBUG
-    fprintf(stderr, "arrangeLine: column=%d, cursorX=%d, visualpos=%d, pos=%d, len=%d\n",
-	    buf->currentColumn, buf->cursorX, buf->visualpos,
-	    buf->pos, buf->currentLine->len);
+    fprintf(stderr,
+	    "arrangeLine: column=%d, cursorX=%d, visualpos=%d, pos=%d, len=%d\n",
+	    buf->currentColumn, buf->cursorX, buf->visualpos, buf->pos,
+	    buf->currentLine->len);
 #endif
 }
 
 void
-cursorXY(Buffer * buf, int x, int y)
+cursorXY(Buffer *buf, int x, int y)
 {
     int oldX;
 
@@ -1032,19 +1089,19 @@ cursorXY(Buffer * buf, int x, int y)
 
     if (buf->cursorX > x) {
 	while (buf->cursorX > x)
-           cursorLeft(buf, COLS / 2);
+	    cursorLeft(buf, buf->COLS / 2);
     }
     else if (buf->cursorX < x) {
 	while (buf->cursorX < x) {
 	    oldX = buf->cursorX;
 
-           cursorRight(buf, COLS / 2);
+	    cursorRight(buf, buf->COLS / 2);
 
 	    if (oldX == buf->cursorX)
 		break;
 	}
 	if (buf->cursorX > x)
-           cursorLeft(buf, COLS / 2);
+	    cursorLeft(buf, buf->COLS / 2);
     }
 }
 
