@@ -1,4 +1,4 @@
-/* $Id: etc.c,v 1.8.2.31 2001/01/22 12:46:55 okabe Exp $ */
+/* $Id: etc.c,v 1.12 2001/12/10 15:42:38 ukai Exp $ */
 #include "fm.h"
 #include <pwd.h>
 #include "myctype.h"
@@ -7,21 +7,36 @@
 #include "hash.h"
 #include "terms.h"
 
-#ifdef GETCWD
+#ifdef HAVE_GETCWD		/* ??? ukai */
 #include <unistd.h>
 #include <sys/param.h>
-#endif				/* GETCWD */
+#endif				/* HAVE_GETCWD */
 
 #include <sys/types.h>
 #include <time.h>
 #include <sys/wait.h>
+#include <signal.h>
 
 #ifdef	__WATT32__
 #define	read(a,b,c)	read_s(a,b,c)
 #define	close(x)	close_s(x)
 #endif				/* __WATT32__ */
 
-#ifndef STRCASECMP
+
+#ifndef HAVE_STRCHR
+char *
+strchr(char *s, char c)
+{
+    while (*s) {
+	if (*s == c)
+	    return s;
+	s++;
+    }
+    return NULL;
+}
+#endif				/* not HAVE_STRCHR */
+
+#ifndef HAVE_STRCASECMP
 int
 strcasecmp(char *s1, char *s2)
 {
@@ -54,24 +69,10 @@ strncasecmp(char *s1, char *s2, int n)
 	return x;
     return 0;
 }
-#endif				/* not STRCASECMP */
+#endif				/* not HAVE_STRCASECMP */
 
 int
-arg_is(char *str, char *tag)
-{
-    while (*tag) {
-	if (tolower(*tag) != tolower(*str))
-	    return 0;
-	tag++;
-	str++;
-    }
-    while (*str && (*str == ' ' || *str == '\t'))
-	str++;
-    return (*str == '=');
-}
-
-int
-columnSkip(Buffer * buf, int offset)
+columnSkip(Buffer *buf, int offset)
 {
     int i, maxColumn;
     int column = buf->currentColumn + offset;
@@ -79,15 +80,13 @@ columnSkip(Buffer * buf, int offset)
     Line *l;
 
     maxColumn = 0;
-    for (i = 0, l = buf->topLine;
-	 i < nlines && l != NULL;
-	 i++, l = l->next) {
+    for (i = 0, l = buf->topLine; i < nlines && l != NULL; i++, l = l->next) {
 	if (l->width < 0)
 	    l->width = COLPOS(l, l->len);
 	if (l->width - 1 > maxColumn)
 	    maxColumn = l->width - 1;
     }
-    maxColumn -= COLS - 1;
+    maxColumn -= buf->COLS - 1;
     if (column < maxColumn)
 	maxColumn = column;
     if (maxColumn < 0)
@@ -100,7 +99,7 @@ columnSkip(Buffer * buf, int offset)
 }
 
 int
-columnPos(Line * line, int column)
+columnPos(Line *line, int column)
 {
     int i;
 
@@ -117,20 +116,22 @@ columnPos(Line * line, int column)
 }
 
 Line *
-lineSkip(Buffer * buf, Line * line, int offset, int last)
+lineSkip(Buffer *buf, Line *line, int offset, int last)
 {
     int i;
     Line *l;
 
     l = currentLineSkip(buf, line, offset, last);
-    for (i = (LASTLINE - 1) - (buf->lastLine->linenumber - l->linenumber);
-	i > 0 && l->prev != NULL;
-	i--, l = l->prev);
+#ifdef NEXTPAGE_TOPLINE
+    if (!nextpage_topline)
+#endif
+	for (i = (LASTLINE - 1) - (buf->lastLine->linenumber - l->linenumber);
+	     i > 0 && l->prev != NULL; i--, l = l->prev) ;
     return l;
 }
 
 Line *
-currentLineSkip(Buffer * buf, Line * line, int offset, int last)
+currentLineSkip(Buffer *buf, Line *line, int offset, int last)
 {
     int i, n;
     Line *l = line;
@@ -140,7 +141,7 @@ currentLineSkip(Buffer * buf, Line * line, int offset, int last)
 	if (buf->lastLine->linenumber < n)
 	    getNextPage(buf, n - buf->lastLine->linenumber);
 	while ((last || (buf->lastLine->linenumber < n)) &&
-	       (getNextPage(buf, 1) != NULL));
+	       (getNextPage(buf, 1) != NULL)) ;
 	if (last)
 	    l = buf->lastLine;
     }
@@ -148,136 +149,51 @@ currentLineSkip(Buffer * buf, Line * line, int offset, int last)
     if (offset == 0)
 	return l;
     if (offset > 0)
-	for (i = 0; i < offset && l->next != NULL; i++, l = l->next);
+	for (i = 0; i < offset && l->next != NULL; i++, l = l->next) ;
     else
-	for (i = 0; i < -offset && l->prev != NULL; i++, l = l->prev);
+	for (i = 0; i < -offset && l->prev != NULL; i++, l = l->prev) ;
     return l;
 }
 
 #define MAX_CMD_LEN 128
 
-static int
-get_cmd(Hash_si * hash,
-	char **s,
-	char *args,
-	char termchar,
-	int defaultcmd,
-	int allow_space,
-	int *status)
+int
+gethtmlcmd(char **s)
 {
+    extern Hash_si tagtable;
     char cmdstr[MAX_CMD_LEN];
     char *p = cmdstr;
     char *save = *s;
     int cmd;
-    if (status)
-	*status = 0;
+
     (*s)++;
     /* first character */
     if (IS_ALNUM(**s) || **s == '_' || **s == '/')
 	*(p++) = tolower(*((*s)++));
     else
-	return defaultcmd;
+	return HTML_UNKNOWN;
     if (p[-1] == '/')
 	SKIP_BLANKS(*s);
-    while ((IS_ALNUM(**s) || **s == '_') &&
-	   p - cmdstr < MAX_CMD_LEN) {
+    while ((IS_ALNUM(**s) || **s == '_') && p - cmdstr < MAX_CMD_LEN) {
 	*(p++) = tolower(*((*s)++));
     }
     if (p - cmdstr == MAX_CMD_LEN) {
 	/* buffer overflow: perhaps caused by bad HTML source */
 	*s = save + 1;
-	if (status)
-	    *status = -1;
-	return defaultcmd;
+	return HTML_UNKNOWN;
     }
     *p = '\0';
 
     /* hash search */
-    cmd = getHash_si(hash, cmdstr, defaultcmd);
-    if (args != NULL) {
-	p = args;
-	while (**s == ' ' || **s == '\t')
-	    (*s)++;
-	while (**s && **s != '\n' && **s != '\r' && **s != termchar) {
-	    *(p++) = *((*s)++);
-	}
-	*p = '\0';
-    }
-    else if (allow_space) {
-	while (**s && **s != termchar)
-	    (*s)++;
-    }
-    if (**s == termchar)
+    cmd = getHash_si(&tagtable, cmdstr, HTML_UNKNOWN);
+    while (**s && **s != '>')
+	(*s)++;
+    if (**s == '>')
 	(*s)++;
-    else if (status)
-	*status = -1;
     return cmd;
 }
 
-int
-gethtmlcmd(char **s, int *status)
-{
-    extern Hash_si tagtable;
-    return get_cmd(&tagtable, s, NULL, '>', HTML_UNKNOWN, TRUE, status);
-}
-
-static char *
-searchAnchorArg(char **arg)
-{
-    char *p;
-    if (**arg == '"') {
-	(*arg)++;
-	p = *arg;
-	while (*p && *p != '"')
-	    p++;
-    }
-    else {
-	p = *arg;
-	while (*p && *p != '>' && *p != ' ' && *p != ',' &&
-	       *p != '\t' && *p != '\n')
-	    p++;
-    }
-    return p;
-}
-
-char *
-getAnchor(char *arg, char **arg_return)
-{
-    char *p;
-    char buf[LINELEN];
-
-    if (arg_is(arg, "name")) {
-	arg += 4;
-	while (*arg && (*arg == ' ' || *arg == '\t'))
-	    arg++;
-	if (*arg != '=')
-	    return NULL;
-	arg++;
-	while (*arg && (*arg == ' ' || *arg == '\t'))
-	    arg++;
-	p = searchAnchorArg(&arg);
-	buf[0] = '#';
-	strncpy(&buf[1], arg, p - arg);
-	if (arg_return)
-	    *arg_return = p;
-	return allocStr(buf, p - arg + 1);
-    }
-    while (*arg && *arg != '=')
-	arg++;
-    if (*arg == '\0')
-	return NULL;
-    arg++;
-    while (*arg && (*arg == ' ' || *arg == '\t'))
-	arg++;
-    p = searchAnchorArg(&arg);
-    if (arg_return)
-	*arg_return = p;
-    if (p == arg)
-	return allocStr(" ", 1);
-    return allocStr(arg, p - arg);
-}
-
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 static int
 parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
 {
@@ -286,11 +202,10 @@ parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
     Linecolor c = *color;
     int i;
 
-    if (*p != ESC_CODE || *(p+1) != '[')
+    if (*p != ESC_CODE || *(p + 1) != '[')
 	return 0;
     p += 2;
-    for (q = p; IS_DIGIT(*q) || *q == ';'; q++)
-	;
+    for (q = p; IS_DIGIT(*q) || *q == ';'; q++) ;
     if (*q != 'm')
 	return 0;
     *str = q + 1;
@@ -302,8 +217,7 @@ parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
 	}
 	if (IS_DIGIT(*p)) {
 	    q = p;
-	    for (p++; IS_DIGIT(*p); p++)
-		;
+	    for (p++; IS_DIGIT(*p); p++) ;
 	    i = atoi(allocStr(q, p - q));
 	    switch (i) {
 	    case 0:
@@ -320,7 +234,7 @@ parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
 	    case 7:
 		e = PE_STAND;
 		break;
-	    case 100:	/* for EWS4800 kterm */
+	    case 100:		/* for EWS4800 kterm */
 		c = 0;
 		break;
 	    case 39:
@@ -338,12 +252,13 @@ parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
 	    }
 	    if (*p == 'm')
 		break;
-	} else {
+	}
+	else {
 	    e = PE_NORMAL;
 	    c = 0;
 	    break;
 	}
-	p++;	/* *p == ';' */
+	p++;			/* *p == ';' */
     }
     *effect = e;
     *color = c;
@@ -356,9 +271,9 @@ parse_ansi_color(char **str, Lineprop *effect, Linecolor *color)
  */
 
 Str
-checkType(Str s, Lineprop * oprop,
-#ifdef ANSI_COLOR
-	  Linecolor * ocolor, int * check_color,
+checkType(Str s, Lineprop *oprop,
+#ifdef USE_ANSI_COLOR
+	  Linecolor *ocolor, int *check_color,
 #endif
 	  int len)
 {
@@ -366,7 +281,7 @@ checkType(Str s, Lineprop * oprop,
     Lineprop effect = PE_NORMAL;
     Lineprop *prop = oprop;
     char *str = s->ptr, *endp = &s->ptr[s->length], *bs = NULL;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     Lineprop ceffect = PE_NORMAL;
     Linecolor cmode = 0;
     Linecolor *color = NULL;
@@ -375,13 +290,13 @@ checkType(Str s, Lineprop * oprop,
     int do_copy = FALSE;
     int size = (len < s->length) ? len : s->length;
 
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
     if (check_color)
 	*check_color = FALSE;
 #endif
     if (ShowEffect) {
 	bs = memchr(str, '\b', s->length);
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	if (ocolor) {
 	    es = memchr(str, ESC_CODE, s->length);
 	    if (es)
@@ -389,7 +304,7 @@ checkType(Str s, Lineprop * oprop,
 	}
 #endif
 	if (s->length > size || (bs != NULL)
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	    || (es != NULL)
 #endif
 	    ) {
@@ -454,8 +369,7 @@ checkType(Str s, Lineprop * oprop,
 		else if (*(str + 1) == '\b') {
 #ifdef JP_CHARSET
 		    if (s->length > 1 && CharType(*(prop - 2)) == PC_KANJI1) {
-			if (str + 4 <= endp &&
-			    !strncmp(str - 2, str + 2, 2)) {
+			if (str + 4 <= endp && !strncmp(str - 2, str + 2, 2)) {
 			    *(prop - 1) |= PE_BOLD;
 			    *(prop - 2) |= PE_BOLD;
 			    str += 4;
@@ -469,8 +383,7 @@ checkType(Str s, Lineprop * oprop,
 		    else
 #endif				/* JP_CHARSET */
 		    if (s->length > 0) {
-			if (str + 3 <= endp &&
-			    *(str - 1) == *(str + 2)) {
+			if (str + 3 <= endp && *(str - 1) == *(str + 2)) {
 			    *(prop - 1) |= PE_BOLD;
 			    str += 3;
 			}
@@ -487,8 +400,7 @@ checkType(Str s, Lineprop * oprop,
 		else {
 #ifdef JP_CHARSET
 		    if (s->length > 1 && CharType(*(prop - 2)) == PC_KANJI1) {
-			if (str + 3 <= endp &&
-			    !strncmp(str - 2, str + 1, 2)) {
+			if (str + 3 <= endp && !strncmp(str - 2, str + 1, 2)) {
 			    *(prop - 1) |= PE_BOLD;
 			    *(prop - 2) |= PE_BOLD;
 			    str += 3;
@@ -502,8 +414,7 @@ checkType(Str s, Lineprop * oprop,
 		    else
 #endif				/* JP_CHARSET */
 		    if (s->length > 0) {
-			if (str + 2 <= endp &&
-			    *(str - 1) == *(str + 1)) {
+			if (str + 2 <= endp && *(str - 1) == *(str + 1)) {
 			    *(prop - 1) |= PE_BOLD;
 			    str += 2;
 			}
@@ -521,12 +432,12 @@ checkType(Str s, Lineprop * oprop,
 		    bs = memchr(str, '\b', endp - str);
 		continue;
 	    }
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	    else if (str > bs)
 		bs = memchr(str, '\b', endp - str);
 #endif
 	}
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	if (es != NULL) {
 	    if (str == es) {
 		int ok = parse_ansi_color(&str, &ceffect, &cmode);
@@ -544,7 +455,7 @@ checkType(Str s, Lineprop * oprop,
 #endif
 
 	mode = get_mctype(str);
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	effect |= ceffect;
 #endif
 #ifdef JP_CHARSET
@@ -557,7 +468,7 @@ checkType(Str s, Lineprop * oprop,
 	    }
 	    prop += 2;
 	    str += 2;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	    if (color) {
 		color[0] = cmode;
 		color[1] = cmode;
@@ -573,7 +484,7 @@ checkType(Str s, Lineprop * oprop,
 		Strcat_char(s, *str);
 	    prop++;
 	    str++;
-#ifdef ANSI_COLOR
+#ifdef USE_ANSI_COLOR
 	    if (color) {
 		*color = cmode;
 		color++;
@@ -595,15 +506,15 @@ calcPosition(char *l, Lineprop *pr, int len, int pos, int bpos, int mode)
     if (l == NULL || len == 0)
 	return bpos;
     if (l == prevl && mode == CP_AUTO) {
-        if (pos <= len)
+	if (pos <= len)
 	    return realColumn[pos];
     }
     prevl = l;
     j = bpos;
     for (i = 0;; i++) {
 	realColumn[i] = j;
-        if (i == len)
-            break;
+	if (i == len)
+	    break;
 	if (l[i] == '\t' && pr[i] == PC_CTRL)
 	    j = (j + Tabstop) / Tabstop * Tabstop;
 #ifndef KANJI_SYMBOLS
@@ -634,10 +545,10 @@ lastFileName(char *path)
 	p++;
     }
 
-    return allocStr(q, 0);
+    return allocStr(q, -1);
 }
 
-#ifdef NOBCOPY
+#ifndef HAVE_BCOPY
 void
 bcopy(void *src, void *dest, int len)
 {
@@ -661,7 +572,30 @@ bzero(void *ptr, int len)
     for (i = 0; i < len; i++)
 	*(ptr++) = 0;
 }
-#endif				/* NOBCOPY */
+#endif				/* not HAVE_BCOPY */
+
+#ifdef USE_INCLUDED_SRAND48
+static unsigned long R1 = 0x1234abcd;
+static unsigned long R2 = 0x330e;
+#define A1 0x5deec
+#define A2 0xe66d
+#define C 0xb
+
+void
+srand48(long seed)
+{
+    R1 = (unsigned long)seed;
+    R2 = 0x330e;
+}
+
+long
+lrand48(void)
+{
+    R1 = (A1 * R1 << 16) + A1 * R2 + A2 * R1 + ((A2 * R2 + C) >> 16);
+    R2 = (A2 * R2 + C) & 0xffff;
+    return (long)(R1 >> 1);
+}
+#endif
 
 char *
 mybasename(char *s)
@@ -675,7 +609,7 @@ mybasename(char *s)
 	p++;
     else
 	p = s;
-    return allocStr(p, 0);
+    return allocStr(p, -1);
 }
 
 char *
@@ -697,16 +631,16 @@ mydirname(char *s)
     return allocStr(s, strlen(s) - strlen(p) + 1);
 }
 
-#ifndef STRERROR
+#ifndef HAVE_STRERROR
 char *
 strerror(int errno)
 {
     extern char *sys_errlist[];
     return sys_errlist[errno];
 }
-#endif				/* not STRERROR */
+#endif				/* not HAVE_STRERROR */
 
-#ifndef SYS_ERRLIST
+#ifndef HAVE_SYS_ERRLIST
 char **sys_errlist;
 
 prepare_sys_errlist()
@@ -722,7 +656,7 @@ prepare_sys_errlist()
     for (i = 1; i < n; i++)
 	sys_errlist[i] = strerror(i);
 }
-#endif				/* not SYS_ERRLIST */
+#endif				/* not HAVE_SYS_ERRLIST */
 
 int
 next_status(char c, int *status)
@@ -903,8 +837,7 @@ read_token(Str buf, char **instr, int *status, int pre, int append)
 		*status = prev_status;
 		return 1;
 	    }
-	    if (*status == R_ST_TAG0 &&
-		!REALLY_THE_BEGINNING_OF_A_TAG(p)) {
+	    if (*status == R_ST_TAG0 && !REALLY_THE_BEGINNING_OF_A_TAG(p)) {
 		/* it seems that this '<' is not a beginning of a tag */
 		Strcat_charp(buf, "&lt;");
 		*status = R_ST_NORMAL;
@@ -945,7 +878,7 @@ correct_irrtag(int status)
 
     while (status != R_ST_NORMAL) {
 	switch (status) {
-	case R_ST_CMNT:		/* required "-->" */
+	case R_ST_CMNT:	/* required "-->" */
 	case R_ST_NCMNT1:	/* required "->" */
 	    c = '-';
 	    break;
@@ -1020,7 +953,7 @@ add_auth_cookie(char *host, char *realm, Str cookie)
 
 /* get last modified time */
 char *
-last_modified(Buffer * buf)
+last_modified(Buffer *buf)
 {
     TextListItem *ti;
     struct stat st;
@@ -1041,12 +974,10 @@ last_modified(Buffer * buf)
     return "unknown";
 }
 
-static char roman_num1[] =
-{
+static char roman_num1[] = {
     'i', 'x', 'c', 'm', '*',
 };
-static char roman_num5[] =
-{
+static char roman_num5[] = {
     'v', 'l', 'd', '*',
 };
 
@@ -1123,36 +1054,32 @@ romanAlphabet(int n)
     return r;
 }
 
-Str
-quoteShell(char *string)
-{
-    Str str = Strnew();
-
-    while (*string != '\0') {
-	if (!IS_ALNUM(*string) && *string != '_' && *string != '.' &&
-	    *string != ':' && *string != '/')
-	    Strcat_char(str, '\\');
-	Strcat_char(str, *(string++));
-    }
-    return str;
-}
-
 void
 mySystem(char *command, int background)
 {
-#ifdef __EMX__ /* jsawa */
-    if (background){
+    if (background) {
+#ifndef HAVE_SETPGRP
 	Str cmd = Strnew_charp("start /f ");
 	Strcat_charp(cmd, command);
 	system(cmd->ptr);
-    }else
-	system(command);
 #else
-    Str cmd = Strnew_charp(command);
-    if (background)
-	cmd = Sprintf("(%s) >/dev/null 2>&1 &", cmd->ptr);
-    system(cmd->ptr);
+	int pid;
+	flush_tty();
+	if ((pid = fork()) == 0) {
+#ifdef SIGCHLD
+	    signal(SIGCHLD, SIG_IGN);
 #endif
+	    setpgrp();
+	    close_tty();
+	    fclose(stdout);
+	    fclose(stderr);
+	    execl("/bin/sh", "sh", "-c", command, NULL);
+	    exit(127);
+	}
+#endif
+    }
+    else
+	system(command);
 }
 
 char *
@@ -1189,12 +1116,63 @@ expandName(char *name)
     return extpath->ptr;
 }
 
-static char *tmpf_base[MAX_TMPF_TYPE] =
+char *
+file_to_url(char *file)
 {
-    "tmp", "src", "frame", "cache" 
+    Str tmp;
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+    char *drive = NULL;
+#endif
+#ifdef SUPPORT_NETBIOS_SHARE
+    char *host = NULL;
+#endif
+
+    file = expandName(file);
+#ifdef SUPPORT_NETBIOS_SHARE
+    if (file[0] == '/' && file[1] == '/') {
+	char *p;
+	file += 2;
+	if (*file) {
+	    p = strchr(file, '/');
+	    if (p != NULL && p != file) {
+		host = allocStr(file, (p - file));
+		file = p;
+	    }
+	}
+    }
+#endif
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+    if (IS_ALPHA(file[0]) && file[1] == ':') {
+	drive = allocStr(file, 2);
+	file += 2;
+    }
+    else
+#endif
+    if (file[0] != '/') {
+	tmp = Strnew_charp(CurrentDir);
+	if (Strlastchar(tmp) != '/')
+	    Strcat_char(tmp, '/');
+	Strcat_charp(tmp, file);
+	file = tmp->ptr;
+    }
+    tmp = Strnew_charp("file://");
+#ifdef SUPPORT_NETBIOS_SHARE
+    if (host)
+	Strcat_charp(tmp, host);
+#endif
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+    if (drive)
+	Strcat_charp(tmp, drive);
+#endif
+    Strcat_charp(tmp, file_quote(cleanupName(file)));
+    return tmp->ptr;
+}
+
+static char *tmpf_base[MAX_TMPF_TYPE] = {
+    "tmp", "src", "frame", "cache"
 };
 static unsigned int tmpf_seq[MAX_TMPF_TYPE];
-    
+
 Str
 tmpfname(int type, char *ext)
 {
@@ -1202,16 +1180,12 @@ tmpfname(int type, char *ext)
     tmpf = Sprintf("%s/w3m%s%d-%d%s",
 		   rc_dir,
 		   tmpf_base[type],
-		   (int) getpid(),
-		   tmpf_seq[type]++,
-		   (ext)? ext : ""
-		   );
+		   (int)getpid(), tmpf_seq[type]++, (ext) ? ext : "");
     return tmpf;
 }
 
 #ifdef USE_COOKIE
-static char *monthtbl[] =
-{
+static char *monthtbl[] = {
     "Jan", "Feb", "Mar", "Apr", "May", "Jun",
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 };
@@ -1329,8 +1303,7 @@ get_time(char **s, int *hour, int *min, int *sec)
     *sec = atoi(tmp->ptr);
 
     if (*hour < 0 || *hour >= 24 ||
-	*min < 0 || *min >= 60 ||
-	*sec < 0 || *sec >= 60) {
+	*min < 0 || *min >= 60 || *sec < 0 || *sec >= 60) {
 	*s = ss;
 	return -1;
     }
@@ -1418,9 +1391,7 @@ mymktime(char *timestr)
     day += (year - 1968) * 1461 / 4;
     day += ((((mon * 153) + 2) / 5) - 672);
     return (time_t) ((day * 60 * 60 * 24) +
-		     (hour * 60 * 60) +
-		     (min * 60) +
-		     sec);
+		     (hour * 60 * 60) + (min * 60) + sec);
 }
 
 #ifdef INET6
@@ -1443,7 +1414,7 @@ FQDN(char *host)
     if (strcasecmp(host, "localhost") == 0)
 	return host;
 
-    for (p = host; *p && *p != '.'; p++);
+    for (p = host; *p && *p != '.'; p++) ;
 
     if (*p == '.')
 	return host;
@@ -1452,7 +1423,7 @@ FQDN(char *host)
     if (!(entry = gethostbyname(host)))
 	return NULL;
 
-    return allocStr(entry->h_name, 0);
+    return allocStr(entry->h_name, -1);
 #else				/* INET6 */
     for (af = ai_family_order_table[DNS_order];; af++) {
 	int error;
