@@ -1,9 +1,9 @@
-/* $Id: linein.c,v 1.1.1.1.4.15 2000/12/07 09:14:14 okabe Exp $ */
+/* $Id: linein.c,v 1.17 2001/12/12 17:04:21 ukai Exp $ */
 #include "fm.h"
 #include "local.h"
 #include "myctype.h"
 
-#ifdef MOUSE
+#ifdef USE_MOUSE
 #ifdef USE_GPM
 #include <gpm.h>
 #endif
@@ -11,7 +11,7 @@
 extern int do_getch();
 #define getch()	do_getch()
 #endif				/* USE_GPM */
-#endif				/* MOUSE */
+#endif				/* USE_MOUSE */
 
 #ifdef __EMX__
 #include <sys/kbdscan.h>
@@ -33,13 +33,13 @@ static int NCFileBuf;
 static int NCFileOffset;
 
 static void insertself(char c),
-  _mvR(void), _mvL(void), _mvRw(void), _mvLw(void), delC(void), insC(void),
-  _mvB(void), _mvE(void), _enter(void), _quo(void), _bs(void), _bsw(void),
-  killn(void), killb(void), _inbrk(void), _esc(void),
-  _prev(void), _next(void), _compl(void), _rcompl(void), _tcompl(void),
-  _dcompl(void), _rdcompl(void);
-#ifdef _EMX_
-static int getcntrl();
+_mvR(void), _mvL(void), _mvRw(void), _mvLw(void), delC(void), insC(void),
+_mvB(void), _mvE(void), _enter(void), _quo(void), _bs(void), _bsw(void),
+killn(void), killb(void), _inbrk(void), _esc(void),
+_prev(void), _next(void), _compl(void), _tcompl(void),
+_dcompl(void), _rdcompl(void), _rcompl(void);
+#ifdef __EMX__
+static int getcntrl(void);
 #endif
 
 static int terminated(unsigned char c);
@@ -49,6 +49,7 @@ static void next_compl(int next);
 static void next_dcompl(int next);
 static Str doComplete(Str ifn, int *status, int next);
 
+/* *INDENT-OFF* */
 void (*InputKeymap[32]) () = {
 /*  C-@     C-a     C-b     C-c     C-d     C-e     C-f     C-g     */
     _compl, _mvB,   _mvL,   _inbrk, delC,   _mvE,   _mvR,   _inbrk,
@@ -59,8 +60,9 @@ void (*InputKeymap[32]) () = {
 /*  C-x     C-y     C-z     C-[     C-\     C-]     C-^     C-_     */
     _tcompl,_mvRw,  iself,  _esc,   iself,  iself,  iself,  iself,
 };
+/* *INDENT-ON* */
 
-static int setStrType(Str str, Lineprop * prop);
+static int setStrType(Str str, Lineprop *prop);
 static void addPasswd(char *p, Lineprop *pr, int len, int pos, int limit);
 static void addStr(char *p, Lineprop *pr, int len, int pos, int limit);
 
@@ -79,7 +81,7 @@ static void ins_kanji(Str tmp);
 #endif
 
 char *
-inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
+inputLineHist(char *prompt, char *def_str, int flag, Hist *hist)
 {
     int opos, x, y, lpos, rpos, epos;
     unsigned char c;
@@ -100,19 +102,23 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
     if (hist != NULL) {
 	use_hist = TRUE;
 	strCurrentBuf = NULL;
-    } else {
+    }
+    else {
 	use_hist = FALSE;
     }
     if (flag & IN_URL) {
 	cm_mode = CPL_ALWAYS | CPL_URL;
 	move_word = TRUE;
-    } else if (flag & IN_FILENAME) {
+    }
+    else if (flag & IN_FILENAME) {
 	cm_mode = CPL_ALWAYS;
 	move_word = TRUE;
-    } else if (flag & IN_PASSWORD) {
+    }
+    else if (flag & IN_PASSWORD) {
 	cm_mode = CPL_NEVER;
 	is_passwd = TRUE;
-    } else if (flag & IN_COMMAND)
+    }
+    else if (flag & IN_COMMAND)
 	cm_mode = CPL_ON;
     else
 	cm_mode = CPL_OFF;
@@ -127,7 +133,8 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
     if (def_str) {
 	strBuf = Strnew_charp(def_str);
 	CLen = CPos = setStrType(strBuf, strProp);
-    } else {
+    }
+    else {
 	strBuf = Strnew();
 	CLen = CPos = 0;
     }
@@ -146,7 +153,8 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 		offset = x - rpos;
 	    else if (y - epos > 0)
 		offset = y - epos;
-	} else if (x - lpos < offset) {
+	}
+	else if (x - lpos < offset) {
 	    if (x - lpos > 0)
 		offset = x - lpos;
 	    else
@@ -162,7 +170,7 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 	move(LASTLINE, opos + x - offset);
 	refresh();
 
-    next_char:
+      next_char:
 	c = getch();
 #ifdef __EMX__
 	if (c == 0) {
@@ -171,7 +179,7 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 	}
 #endif
 	cm_clear = TRUE;
- 	cm_disp_clear = TRUE;
+	cm_disp_clear = TRUE;
 #ifdef JP_CHARSET
 	if (mode == PC_KANJI1) {
 	    mode = PC_KANJI2;
@@ -179,31 +187,37 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 		goto next_char;
 	    Strcat_char(tmp, (c | (DisplayCode == CODE_SJIS ? 0 : 0x80)));
 	    tmp = conv_str(tmp,
-		(DisplayCode == CODE_SJIS ? CODE_SJIS : CODE_EUC), InnerCode);
+			   (DisplayCode == CODE_SJIS ? CODE_SJIS : CODE_EUC),
+			   InnerCode);
 	    ins_kanji(tmp);
 	}
 	else
 #endif
 	if (!i_quote &&
-	    (((cm_mode & CPL_ALWAYS) && (c == CTRL_I || c == ' ')) ||
-	     ((cm_mode & CPL_ON) && (c == CTRL_I)))) {
+		(((cm_mode & CPL_ALWAYS) && (c == CTRL_I || c == ' ')) ||
+		     ((cm_mode & CPL_ON) && (c == CTRL_I)))) {
 #ifdef EMACS_LIKE_LINEEDIT
-	    if (cm_next) {
+	    if (emacs_like_lineedit && cm_next) {
 		_dcompl();
 		need_redraw = TRUE;
 	    }
 	    else {
+#endif
 		_compl();
 		cm_disp_next = -1;
+#ifdef EMACS_LIKE_LINEEDIT
 	    }
-#else
-	    _compl();
-	    cm_disp_next = -1;
+#endif
 	}
 	else if (!i_quote && CLen == CPos &&
 		 (cm_mode & CPL_ALWAYS || cm_mode & CPL_ON) && c == CTRL_D) {
-	    _dcompl();
-	    need_redraw = TRUE;
+#ifdef EMACS_LIKE_LINEEDIT
+	    if (!emacs_like_lineedit) {
+#endif
+		_dcompl();
+		need_redraw = TRUE;
+#ifdef EMACS_LIKE_LINEEDIT
+	    }
 #endif
 	}
 	else if (!i_quote && c == DEL_CODE) {
@@ -212,10 +226,10 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 	    cm_disp_next = -1;
 	}
 	else if (!i_quote && c < 0x20) {	/* Control code */
-	    (*InputKeymap[(int) c]) (c);
+	    (*InputKeymap[(int)c]) (c);
 	    if (cm_clear)
 		cm_next = FALSE;
- 	    if (cm_disp_clear)
+	    if (cm_disp_clear)
 		cm_disp_next = -1;
 	}
 #ifdef JP_CHARSET
@@ -256,6 +270,8 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
 	    CPos++;
 	    mode = PC_ASCII;
 	}
+	if (CLen && (flag & IN_CHAR))
+	    break;
     } while (i_cont);
 
     if (need_redraw)
@@ -271,13 +287,13 @@ inputLineHist(char *prompt, char *def_str, int flag, Hist * hist)
     }
     if (use_hist && !(flag & IN_URL) && *p != '\0') {
 	char *q = lastHist(hist);
-	if (! q || strcmp(q, p))
+	if (!q || strcmp(q, p))
 	    pushHist(hist, p);
     }
     if (flag & IN_FILENAME)
 	return expandName(p);
     else
-	return allocStr(p, 0);
+	return allocStr(p, -1);
 }
 
 #ifdef __EMX__
@@ -288,7 +304,7 @@ getcntrl(void)
     case K_DEL:
 	return CTRL_D;
     case K_LEFT:
-	retrun CTRL_B;
+	return CTRL_B;
     case K_RIGHT:
 	return CTRL_F;
     case K_UP:
@@ -316,7 +332,7 @@ addPasswd(char *p, Lineprop *pr, int len, int offset, int limit)
     int rcol = 0, ncol;
 
     ncol = calcPosition(p, pr, len, len, 0, CP_AUTO);
-    if (ncol> offset + limit)
+    if (ncol > offset + limit)
 	ncol = offset + limit;
     if (offset) {
 	addChar('{', 0);
@@ -357,17 +373,19 @@ addStr(char *p, Lineprop *pr, int len, int offset, int limit)
 #endif
 	ncol = calcPosition(p, pr, len, i + delta, 0, CP_AUTO);
 	if (ncol - offset > limit)
-	   break;
+	    break;
 	if (p[i] == '\t') {
 	    for (; rcol < ncol; rcol++)
 		addChar(' ', 0);
 	    continue;
 #ifdef JP_CHARSET
-	} else if (delta == 2) {
+	}
+	else if (delta == 2) {
 	    addChar(p[i], pr[i]);
-	    addChar(p[i+1], pr[i+1]);
+	    addChar(p[i + 1], pr[i + 1]);
 #endif
-	} else
+	}
+	else
 	    addChar(p[i], pr[i]);
 	rcol = ncol;
     }
@@ -393,7 +411,10 @@ ins_kanji(Str tmp)
 static void
 _esc(void)
 {
-    char c, c2;
+    char c;
+#ifdef JP_CHARSET
+    char c2;
+#endif
 
     switch (c = getch()) {
     case '[':
@@ -411,30 +432,48 @@ _esc(void)
 	case 'D':
 	    _mvL();
 	    break;
+#if defined(__CYGWIN__) && defined(USE_MOUSE)
+	case 'M':
+	    if ((is_xterm & (NEED_XTERM_ON | NEED_XTERM_OFF)) == NEED_XTERM_ON) {
+		getch();
+		getch();
+		getch();
+	    }
+	    break;
+#endif
 	}
 	break;
     case CTRL_I:
     case ' ':
 #ifdef EMACS_LIKE_LINEEDIT
-	_rdcompl();
-	cm_clear = FALSE;
-#else
-	_rcompl();
+	if (emacs_like_lineedit) {
+	    _rdcompl();
+	    cm_clear = FALSE;
+	    need_redraw = TRUE;
+	}
+	else
+#endif
+	    _rcompl();
 	break;
     case CTRL_D:
-	_rdcompl();
+#ifdef EMACS_LIKE_LINEEDIT
+	if (!emacs_like_lineedit)
 #endif
+	    _rdcompl();
 	need_redraw = TRUE;
 	break;
 #ifdef EMACS_LIKE_LINEEDIT
     case 'f':
-	_mvRw();
+	if (emacs_like_lineedit)
+	    _mvRw();
 	break;
     case 'b':
-	_mvLw();
+	if (emacs_like_lineedit)
+	    _mvLw();
 	break;
     case CTRL_H:
-	_bsw();
+	if (emacs_like_lineedit)
+	    _bsw();
 	break;
 #endif
 #ifdef JP_CHARSET
@@ -498,14 +537,12 @@ static void
 _mvLw(void)
 {
     int first = 1;
-    while(CPos > 0
-	  && ( first || !terminated(strBuf->ptr[CPos-1])))
-    {
+    while (CPos > 0 && (first || !terminated(strBuf->ptr[CPos - 1]))) {
 	CPos--;
 	first = 0;
 #ifdef JP_CHARSET
-    if (strProp[CPos] == PC_KANJI2)
-	CPos--;
+	if (strProp[CPos] == PC_KANJI2)
+	    CPos--;
 #endif				/* JP_CHARSET */
 	if (!move_word)
 	    break;
@@ -515,18 +552,16 @@ _mvLw(void)
 static void
 _mvRw(void)
 {
-  int first = 1;
-    while(CPos < CLen
-	  && ( first || !terminated(strBuf->ptr[CPos-1])))
-    {
+    int first = 1;
+    while (CPos < CLen && (first || !terminated(strBuf->ptr[CPos - 1]))) {
 	CPos++;
 	first = 0;
 #ifdef JP_CHARSET
-    if (strProp[CPos] == PC_KANJI2)
-	CPos++;
+	if (strProp[CPos] == PC_KANJI2)
+	    CPos++;
 #endif				/* JP_CHARSET */
-	if(!move_word)
-	  break;
+	if (!move_word)
+	    break;
     }
 }
 
@@ -554,9 +589,9 @@ static void
 _bsw(void)
 {
     int t = 0;
-    while(CPos > 0 && !t) {
+    while (CPos > 0 && !t) {
 	_mvL();
-	t = (move_word && terminated(strBuf->ptr[CPos-1]));
+	t = (move_word && terminated(strBuf->ptr[CPos - 1]));
 	delC();
     }
 }
@@ -650,11 +685,17 @@ next_compl(int next)
 	return;
     cm_clear = FALSE;
     if (!cm_next) {
-	for (b = CPos - 1; b >= 0; b--) {
-	    if (strBuf->ptr[b] == ' ' || strBuf->ptr[b] == CTRL_I)
-		break;
+	if (cm_mode & CPL_ALWAYS) {
+	    b = 0;
+	}
+	else {
+	    for (b = CPos - 1; b >= 0; b--) {
+		if ((strBuf->ptr[b] == ' ' || strBuf->ptr[b] == CTRL_I) &&
+		    !((b > 0) && strBuf->ptr[b - 1] == '\\'))
+		    break;
+	    }
+	    b++;
 	}
-	b++;
 	a = CPos;
 	CBeforeBuf = Strsubstr(strBuf, 0, b);
 	buf = Strsubstr(strBuf, b, a - b);
@@ -710,10 +751,12 @@ next_dcompl(int next)
     if (LASTLINE >= 3) {
 	comment = TRUE;
 	nline = LASTLINE - 2;
-    } else if (LASTLINE) {
+    }
+    else if (LASTLINE) {
 	comment = FALSE;
 	nline = LASTLINE;
-    } else {
+    }
+    else {
 	return;
     }
     if (cm_disp_next >= 0) {
@@ -737,7 +780,7 @@ next_dcompl(int next)
 	return;
     cm_disp_next = 0;
 
-    d = Strdup(CDirBuf);
+    d = Str_conv_to_system(Strdup(CDirBuf));
     if (d->length > 0 && Strlastchar(d) != '/')
 	Strcat_char(d, '/');
     if (cm_mode & CPL_URL && d->ptr[0] == 'f') {
@@ -755,25 +798,28 @@ next_dcompl(int next)
     for (i = 0; i < NCFileBuf; i++) {
 	n = strlen(CFileBuf[i]) + 3;
 	if (len < n)
-	   len = n;
+	    len = n;
     }
     col = COLS / len;
     if (col == 0)
 	col = 1;
     row = (NCFileBuf + col - 1) / col;
 
-disp_next:
+  disp_next:
     if (comment) {
 	if (row > nline) {
 	    row = nline;
 	    y = 0;
-	} else
+	}
+	else
 	    y = nline - row + 1;
-    } else {
+    }
+    else {
 	if (row >= nline) {
 	    row = nline;
 	    y = 0;
-	} else
+	}
+	else
 	    y = nline - row - 1;
     }
     if (y) {
@@ -797,7 +843,7 @@ disp_next:
 	    clrtoeolx();
 	    f = Strdup(d);
 	    Strcat_charp(f, CFileBuf[n]);
-	    addstr(CFileBuf[n]);
+	    addstr(conv_from_system(CFileBuf[n]));
 	    if (stat(expandName(f->ptr), &st) != -1 && S_ISDIR(st.st_mode))
 		addstr("/");
 	}
@@ -808,14 +854,62 @@ disp_next:
 	clrtoeolx();
 	bold();
 #ifdef EMACS_LIKE_LINEEDIT
-	addstr("----- Press TAB to continue -----");
-#else
-	addstr("----- Press CTRL-D to continue -----");
+	if (emacs_like_lineedit)
+	    addstr("----- Press TAB to continue -----");
+	else
 #endif
+	    addstr("----- Press CTRL-D to continue -----");
 	boldend();
     }
 }
 
+
+Str
+escape_spaces(Str s)
+{
+    Str tmp = NULL;
+    char *p;
+
+    if (s == NULL)
+	return s;
+    for (p = s->ptr; *p; p++) {
+	if (*p == ' ' || *p == CTRL_I) {
+	    if (tmp == NULL)
+		tmp = Strnew_charp_n(s->ptr, (int)(p - s->ptr));
+	    Strcat_char(tmp, '\\');
+	}
+	if (tmp)
+	    Strcat_char(tmp, *p);
+    }
+    if (tmp)
+	return tmp;
+    return s;
+}
+
+
+Str
+unescape_spaces(Str s)
+{
+    Str tmp = NULL;
+    char *p;
+
+    if (s == NULL)
+	return s;
+    for (p = s->ptr; *p; p++) {
+	if (*p == '\\' && (*(p + 1) == ' ' || *(p + 1) == CTRL_I)) {
+	    if (tmp == NULL)
+		tmp = Strnew_charp_n(s->ptr, (int)(p - s->ptr));
+	}
+	else {
+	    if (tmp)
+		Strcat_char(tmp, *p);
+	}
+    }
+    if (tmp)
+	return tmp;
+    return s;
+}
+
 static Str
 doComplete(Str ifn, int *status, int next)
 {
@@ -827,23 +921,25 @@ doComplete(Str ifn, int *status, int next)
 
     if (!cm_next) {
 	NCFileBuf = 0;
+	ifn = Str_conv_to_system(ifn);
+	if (cm_mode & CPL_ON)
+	    ifn = unescape_spaces(ifn);
 	CompleteBuf = Strdup(ifn);
-	while (Strlastchar(CompleteBuf) != '/' &&
-	       CompleteBuf->length > 0)
+	while (Strlastchar(CompleteBuf) != '/' && CompleteBuf->length > 0)
 	    Strshrink(CompleteBuf, 1);
 	CDirBuf = Strdup(CompleteBuf);
 	if (cm_mode & CPL_URL) {
 	    if (strncmp(CompleteBuf->ptr, "file://localhost/", 17) == 0)
 		Strdelete(CompleteBuf, 0, 16);
 	    else if (strncmp(CompleteBuf->ptr, "file:///", 8) == 0)
-		 Strdelete(CompleteBuf, 0, 7);
+		Strdelete(CompleteBuf, 0, 7);
 	    else if (strncmp(CompleteBuf->ptr, "file:/", 6) == 0 &&
 		     CompleteBuf->ptr[6] != '/')
 		Strdelete(CompleteBuf, 0, 5);
 	    else {
 		CompleteBuf = Strdup(ifn);
 		*status = CPL_FAIL;
-		return CompleteBuf;
+		return Str_conv_to_system(CompleteBuf);
 	    }
 	}
 	if (CompleteBuf->length == 0) {
@@ -855,6 +951,8 @@ doComplete(Str ifn, int *status, int next)
 	if ((d = opendir(expandName(CompleteBuf->ptr))) == NULL) {
 	    CompleteBuf = Strdup(ifn);
 	    *status = CPL_FAIL;
+	    if (cm_mode & CPL_ON)
+		CompleteBuf = escape_spaces(CompleteBuf);
 	    return CompleteBuf;
 	}
 	fn = lastFileName(ifn->ptr);
@@ -864,18 +962,20 @@ doComplete(Str ifn, int *status, int next)
 	    dir = readdir(d);
 	    if (dir == NULL)
 		break;
-	    if (fl == 0 && (!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, "..")))
+	    if (fl == 0
+		&& (!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, "..")))
 		continue;
 	    if (!strncmp(dir->d_name, fn, fl)) {	/* match */
 		NCFileBuf++;
 		CFileBuf = New_Reuse(char *, CFileBuf, NCFileBuf);
-		CFileBuf[NCFileBuf - 1] = New_N(char, strlen(dir->d_name) + 1);
+		CFileBuf[NCFileBuf - 1] =
+		    NewAtom_N(char, strlen(dir->d_name) + 1);
 		strcpy(CFileBuf[NCFileBuf - 1], dir->d_name);
 		if (NCFileBuf == 1) {
 		    CFileName = Strnew_charp(dir->d_name);
 		}
 		else {
-		    for (i = 0; CFileName->ptr[i] == dir->d_name[i]; i++);
+		    for (i = 0; CFileName->ptr[i] == dir->d_name[i]; i++) ;
 		    Strtruncate(CFileName, i);
 		}
 	    }
@@ -884,6 +984,8 @@ doComplete(Str ifn, int *status, int next)
 	if (NCFileBuf == 0) {
 	    CompleteBuf = Strdup(ifn);
 	    *status = CPL_FAIL;
+	    if (cm_mode & CPL_ON)
+		CompleteBuf = escape_spaces(CompleteBuf);
 	    return CompleteBuf;
 	}
 	qsort(CFileBuf, NCFileBuf, sizeof(CFileBuf[0]), strCmp);
@@ -923,7 +1025,9 @@ doComplete(Str ifn, int *status, int next)
 	if (stat(expandName(p), &st) != -1 && S_ISDIR(st.st_mode))
 	    Strcat_char(CompleteBuf, '/');
     }
-    return CompleteBuf;
+    if (cm_mode & CPL_ON)
+	CompleteBuf = escape_spaces(CompleteBuf);
+    return Str_conv_from_system(CompleteBuf);
 }
 
 static void
@@ -932,13 +1036,14 @@ _prev(void)
     Hist *hist = CurrentHist;
     char *p;
 
-    if (! use_hist)
+    if (!use_hist)
 	return;
     if (strCurrentBuf) {
 	p = prevHist(hist);
 	if (p == NULL)
 	    return;
-    } else {
+    }
+    else {
 	p = lastHist(hist);
 	if (p == NULL)
 	    return;
@@ -955,14 +1060,15 @@ _next(void)
     Hist *hist = CurrentHist;
     char *p;
 
-    if (! use_hist)
+    if (!use_hist)
 	return;
     if (strCurrentBuf == NULL)
 	return;
     p = nextHist(hist);
     if (p) {
 	strBuf = Strnew_charp(p);
-    } else {
+    }
+    else {
 	strBuf = strCurrentBuf;
 	strCurrentBuf = NULL;
     }
@@ -971,7 +1077,7 @@ _next(void)
 }
 
 static int
-setStrType(Str str, Lineprop * prop)
+setStrType(Str str, Lineprop *prop)
 {
     Lineprop ctype;
     int i = 0, delta;
@@ -985,25 +1091,26 @@ setStrType(Str str, Lineprop * prop)
 #ifdef JP_CHARSET
 	if (ctype == PC_KANJI) {
 	    prop[i] = PC_KANJI1;
-	    prop[i+1] = PC_KANJI2;
+	    prop[i + 1] = PC_KANJI2;
 	}
 	else
 #endif
-	prop[i] = ctype;
+	    prop[i] = ctype;
     }
     return i;
 }
 
 static int
-terminated(unsigned char c){
-  int termchar[] = {'/', '&', '?', -1};
-  int *tp;
+terminated(unsigned char c)
+{
+    int termchar[] = { '/', '&', '?', -1 };
+    int *tp;
 
-  for(tp = termchar; *tp > 0; tp++){
-    if(c == *tp){
-      return 1;
+    for (tp = termchar; *tp > 0; tp++) {
+	if (c == *tp) {
+	    return 1;
+	}
     }
-  }
 
-  return 0;
+    return 0;
 }
