@@ -1,8 +1,9 @@
+/* $Id: search.c,v 1.4 2001/11/24 02:01:26 ukai Exp $ */
 #include "fm.h"
 #include "regex.h"
 
 int
-forwardSearch(Buffer * buf, char *str)
+forwardSearch(Buffer *buf, char *str)
 {
     char *p, *first, *last;
     Line *l, *begin;
@@ -14,6 +15,10 @@ forwardSearch(Buffer * buf, char *str)
 	return FALSE;
     }
     l = begin = buf->currentLine;
+    if (l == NULL) {
+	disp_message("Not found", FALSE);
+	return FALSE;
+    }
     pos = buf->pos + 1;
 #ifdef JP_CHARSET
     if (l->propBuf[pos] & PC_KANJI2)
@@ -66,7 +71,7 @@ forwardSearch(Buffer * buf, char *str)
 }
 
 int
-backwardSearch(Buffer * buf, char *str)
+backwardSearch(Buffer *buf, char *str)
 {
     char *p, *q, *found, *first, *last;
     Line *l, *begin;
@@ -78,6 +83,10 @@ backwardSearch(Buffer * buf, char *str)
 	return FALSE;
     }
     l = begin = buf->currentLine;
+    if (l == NULL) {
+	disp_message("Not found", FALSE);
+	return FALSE;
+    }
     if (buf->pos > 0) {
 	pos = buf->pos - 1;
 #ifdef JP_CHARSET
